@@ -17,6 +17,10 @@
 /*	SMTP_TLS_POLICY *tls;
 /*
 /*	void	smtp_tls_policy_cache_flush()
+/*
+/*	int	smtp_tls_authorize_mx_hostname(tls, qname)
+/*	SMTP_TLS_POLICY *tls;
+/*	const char *qname;
 /* DESCRIPTION
 /*	smtp_tls_list_init() initializes lookup tables used by the TLS
 /*	policy engine.
@@ -38,6 +42,11 @@
 /*	smtp_tls_policy_cache_flush() destroys the TLS policy cache
 /*	and contents.
 /*
+/*	smtp_tls_authorize_mx_hostname() authorizes an MX host if the
+/*	name used for host lookup satisfies a TLS policy MX name
+/*	constraint (for example, an STS policy MX pattern), or if the
+/*	TLS policy has no name constraint.
+/*
 /*	Arguments:
 /* .IP why
 /*	A pointer to a DSN_BUF which holds error status information when
@@ -107,6 +116,7 @@
 #include <valid_hostname.h>
 #include <valid_utf8_hostname.h>
 #include <ctable.h>
+#include <midna_domain.h>
 
 /* Global library. */
 
@@ -136,6 +146,51 @@ static void dane_init(SMTP_TLS_POLICY *, SMTP_ITERATOR *);
 static MAPS *tls_policy;		/* lookup table(s) */
 static MAPS *tls_per_site;		/* lookup table(s) */
 
+/* match_sts_mx_host_pattern -  match hostname against STS policy MX pattern */
+
+static int match_sts_mx_host_pattern(const char *pattern, const char *qname)
+{
+    const char *first_dot_in_qname;
+
+    /* Caller guarantees that inputs are in ASCII form. */
+    return (strcasecmp(qname, pattern) == 0
+	    || (pattern[0] == '*' && pattern[1] == '.' && pattern[2] != 0
+		&& (first_dot_in_qname = strchr(qname, '.')) != 0
+		&& first_dot_in_qname > qname
+		&& strcasecmp(first_dot_in_qname + 1, pattern + 2) == 0));
+}
+
+/* smtp_tls_authorize_mx_hostname - enforce applicable MX hostname policy */
+
+int     smtp_tls_authorize_mx_hostname(SMTP_TLS_POLICY *tls, const char *name)
+{
+
+#define SAFE_FOR_SMTP_TLS_ENF_STS_MX_PAT(tls) (var_smtp_tls_enf_sts_mx_pat \
+	    && (tls)->ext_policy_type != 0 \
+	    && strcasecmp((tls)->ext_policy_type, "sts") == 0 \
+	    && (tls)->matchargv != 0 && (tls)->ext_mx_host_patterns != 0)
+
+    /* Enforce STS policy MX patterns. */
+    if (SAFE_FOR_SMTP_TLS_ENF_STS_MX_PAT(tls)) {
+	const char *aname;
+	char  **pattp;
+
+#ifndef NO_EAI
+	if (!allascii(name) && (aname = midna_domain_to_ascii(name)) != 0) {
+	    if (msg_verbose)
+		msg_info("%s asciified to %s", name, aname);
+	} else
+#endif
+	    aname = name;
+	for (pattp = tls->ext_mx_host_patterns->argv; *pattp; pattp++)
+	    if (match_sts_mx_host_pattern(*pattp, aname))
+		return (1);
+	return (0);
+    }
+    /* No applicable policy name patterns. */
+    return (1);
+}
+
 /* smtp_tls_list_init - initialize per-site policy lists */
 
 void    smtp_tls_list_init(void)
@@ -285,12 +340,17 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
     /*
      * Errors in attributes may have security consequences, don't ignore
      * errors that can degrade security.
+     * 
+     * Caution: normalize whitespace, to neutralize line break etc. characters
+     * inside the value portion of { name = value }.
      */
     while ((tok = mystrtokq(&policy, CHARS_COMMA_SP, CHARS_BRACE)) != 0) {
 	const char *err;
 
+#define EXTPAR_OPT	(EXTPAR_FLAG_STRIP | EXTPAR_FLAG_NORMAL_WS)
+
 	if ((tok[0] == CHARS_BRACE[0]
-	     && (err = free_me = extpar(&tok, CHARS_BRACE, EXTPAR_FLAG_STRIP)) != 0)
+	     && (err = free_me = extpar(&tok, CHARS_BRACE, EXTPAR_OPT)) != 0)
 	    || (err = split_nameval(tok, &name, &val)) != 0) {
 	    msg_warn("%s: malformed attribute/value pair \"%s\": %s",
 		     WHERE, tok, err);
@@ -454,6 +514,7 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 	    tls->ext_policy_type = mystrdup(val);
 	    continue;
 	}
+	/* Only one instance per policy. */
 	if (!strcasecmp(name, EXT_POLICY_DOMAIN)) {
 	    if (tls->ext_policy_domain) {
 		msg_warn("%s: attribute \"%s\" is specified multiple times",
@@ -511,6 +572,10 @@ static void tls_policy_lookup_one(SMTP_TLS_POLICY *tls, int *site_level,
 	    INVALID_RETURN(tls->why, site_level);
 	}
     }
+    if (SAFE_FOR_SMTP_TLS_ENF_STS_MX_PAT(tls)) {
+	argv_truncate(tls->matchargv, 0);
+	argv_add(tls->matchargv, "hostname", (char *) 0);
+    }
     FREE_RETURN;
 }
 
@@ -641,11 +706,23 @@ static void *policy_create(const char *unused_key, void *context)
      * Compute the per-site TLS enforcement level. For compatibility with the
      * original TLS patch, this algorithm is gives equal precedence to host
      * and next-hop policies.
+     * 
+     * When "TLS-Required: no" is in effect, skip TLS policy lookup and limit
+     * the security level to "may". Do not reset the security level after
+     * policy lookup, as that would result in errors. For example, when TLSA
+     * records are looked up for security level "dane", and then the security
+     * level is reset to "may", the activation of those TLSA records will
+     * fail.
      */
     tls->level = global_tls_level();
     site_level = TLS_LEV_NOTFOUND;
 
-    if (tls_policy) {
+    if (STATE_TLS_NOT_REQUIRED(iter->parent)) {
+	if (msg_verbose)
+	    msg_info("%s: no tls policy lookup", __func__);
+	if (tls->level > TLS_LEV_MAY)
+	    tls->level = TLS_LEV_MAY;
+    } else if (tls_policy) {
 	tls_policy_lookup(tls, &site_level, dest, "next-hop destination");
     } else if (tls_per_site) {
 	tls_site_lookup(tls, &site_level, dest, "next-hop destination");
